/*
	Copyright (C) 2011  Andrew Cotter

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/**
	\internal
	\file client_data.hh
	\brief ClientData class
*/




#ifndef __CLIENT_DATA_HH__
#define __CLIENT_DATA_HH__

#ifdef __cplusplus




//============================================================================
//    ClientData class
//============================================================================


template< typename t_Type >
struct ClientData {

	inline ClientData() : m_data() {
	}

	inline ClientData( ClientData const& other ) : m_data( other.m_data ) {
	}

	template< typename t_OtherType >
	inline ClientData( t_OtherType const& data ) : m_data( data ) {
	}


	inline t_Type const& Get() const {

		return m_data;
	}

	inline t_Type& Get() {

		return m_data;
	}

	template< typename t_OtherType >
	inline void Set( t_OtherType const& data ) {

		m_data = data;
	}


private:

	t_Type m_data;
};


template< typename t_Type >
struct ClientData< t_Type* > {

	inline ClientData() : m_data( NULL ) {
	}

	inline ClientData( ClientData const& other ) : m_data( other.m_data ) {
	}

	template< typename t_OtherType >
	inline ClientData( t_OtherType const& data ) : m_data( data ) {
	}


	inline t_Type* const Get() const {

		return m_data;
	}

	inline t_Type*& Get() {

		return m_data;
	}

	template< typename t_OtherType >
	inline void Set( t_OtherType const& data ) {

		m_data = data;
	}


private:

	t_Type* m_data;
};


template<>
struct ClientData< void > {
};




#endif    /* __cplusplus */

#endif    /* __CLIENT_DATA_HH__ */
