/*
	Copyright (C) 2011  Andrew Cotter

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/**
	\internal
	\file maybe_threaded.hh
	\brief MaybeThreaded template
*/




#ifndef __MAYBE_THREADED_HH__
#define __MAYBE_THREADED_HH__

#ifdef __cplusplus




#include <boost/thread.hpp>
#include <boost/thread/barrier.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/recursive_mutex.hpp>
#include <boost/thread/locks.hpp>




//============================================================================
//    MaybeThreaded template
//============================================================================


template< bool t_Threaded >
struct MaybeThreaded {

	typedef boost::barrier Barrier;

	typedef boost::mutex Mutex;
	typedef boost::lock_guard< Mutex > Lock;

	typedef boost::recursive_mutex RecursiveMutex;
	typedef boost::lock_guard< RecursiveMutex > RecursiveLock;


private:

	// private and unimplemented
	inline MaybeThreaded();
	inline MaybeThreaded( MaybeThreaded const& );
	inline MaybeThreaded const& operator=( MaybeThreaded const& );
};


template<>
struct MaybeThreaded< false > {

	struct Barrier {

		inline Barrier( unsigned int count ) { BOOST_ASSERT( count == 1 ); }

		inline bool wait() { return true; }
	};


	struct Mutex {

		inline Mutex() { }

		inline void lock() { }
		inline void unlock() { }
	};

	struct Lock {

		inline explicit Lock( Mutex& ) { }
	};


	typedef Mutex RecursiveMutex;
	typedef Lock RecursiveLock;


private:

	// private and unimplemented
	inline MaybeThreaded();
	inline MaybeThreaded( MaybeThreaded const& );
	inline MaybeThreaded const& operator=( MaybeThreaded const& );
};




#endif    /* __cplusplus */

#endif    /* __MAYBE_THREADED_HH__ */
